{-- TerraHS - Interface between TerraLib and Haskell

    (c) Sergio Costa (INPE) - Setembro, 2005

	This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License 2.1 as published by the Free Software Foundation
    (http://www.opensource.org/licenses/gpl-license.php)
--}

{- | A module for supporting the functions of TerraLib TeGeometryAlgorithms

More information - <http://www.terralib.org>
-}
module TerraHS.TerraLib.TeGeometryAlgorithms 
	(
		 tedistance, telength, tearea, setPrecision, tebufferregion 		
	) 
	 where


import Foreign (unsafePerformIO)
import Foreign.C.String
import qualified Foreign.Ptr (Ptr)

--locais
import TerraHS.Algebras.Base.Object
import TerraHS.TerraLib.TeGeometry
import TerraHS.TerraLib.TePoint
import TerraHS.TerraLib.TeLine2D
import TerraHS.TerraLib.TeCell
import TerraHS.TerraLib.TeBox
import TerraHS.TerraLib.TePolygon

import TerraHS.Algebras.Base.Operations
import TerraHS.Algebras.Spatial.Geometries

---------------------------------------
--------- centroid operations ---------
---------------------------------------

			
instance Centroid TePointSet  TePoint where
	centroid ps = unsafePerformIO ( ( new ps ) >>= tepointset_tefindcentroid >>= fromPointer >>= return )
	

instance Centroid TeLineSet TePoint where
	centroid ls = unsafePerformIO ( ( new ls )	>>= telineset_tefindcentroid >>= fromPointer >>= return )

instance Centroid TePolygonSet TePoint where
	centroid pols = unsafePerformIO ( ( new pols )	>>= tepolygonset_tefindcentroid >>= fromPointer >>= return )

instance Centroid TePolygon TePoint where
	centroid pols = unsafePerformIO ( ( new pols )	>>= tepolygon_tefindcentroid >>= fromPointer >>= return )
	
---------------------------------------
--------- metric operations ---------
---------------------------------------
	
-- distance between two points	
tedistance pt1 pt2 = ( unsafePerformIO (applyf pt1 toTE pt2 toTE tepoint_tedistance))
	where
	toTE tx = new tx

-- | Returns the length of a Line 2D.
telength :: TeLine2D -> Double
telength l = ( unsafePerformIO (new l >>= h_telength >>= return ))

-- | Returns the area of a TePolygon 
tearea :: TePolygon -> Double
tearea p = ( unsafePerformIO (new p >>= tegeometryarea >>= return ))


applyf ea fab ec fcd fbde  = fab ea >>= \eb -> fcd ec >>= \ed -> fbde eb ed >>= return


tebufferregion :: TePolygon  -> Double ->  Int -> [TeGeometry]
tebufferregion pol distance npoints = unsafePerformIO (tebufferregion')
	where
	tebufferregion' = do
		ptrpol <- (new pol)
		ptrpolset <- (h_tebufferregion ptrpol distance npoints)
		(polset2geoset ptrpolset)
	
polset2geoset gs = size gs >>= (polygonset2geometryset gs 0)

----------- overlay



instance Set [TeGeometry]   where
	union gs1 gs2 = unsafePerformIO (apply1 gs1 gs2 (\ptr1 ptr2 -> teoverlay ptr1 ptr2 2))
	difference gs1 gs2 = unsafePerformIO (apply1 gs1 gs2 (\ptr1 ptr2 -> teoverlay ptr1 ptr2 1))
	intersection gs1 gs2 = unsafePerformIO (apply1 gs1 gs2 (\ptr1 ptr2 -> teoverlay ptr1 ptr2 4))

instance Set TePolygonSet   where
	union gs1 gs2 = (TePolygonSet (map toPolygon (unsafePerformIO (apply gs1 gs2 (\ptr1 ptr2 -> teoverlay ptr1 ptr2 2)))))
	difference gs1 gs2 = (TePolygonSet (map toPolygon (unsafePerformIO (apply gs1 gs2 (\ptr1 ptr2 -> teoverlay ptr1 ptr2 1)))))
	difference gs1 gs2 = (TePolygonSet (map toPolygon (unsafePerformIO (apply gs1 gs2 (\ptr1 ptr2 -> teoverlay ptr1 ptr2 4)))))
	
  
apply1 gs1 gs2 f =  (new (TePolygonSet (map toPolygon1 gs1))) >>= \ps1 -> (new (TePolygonSet (map toPolygon1 gs2))) >>= \ps2 -> f ps1 ps2 >>= polset2geoset >>= return
	where
	polset2geoset gs = size gs >>= (polygonset2geometryset gs 0)		
	toPolygon1 (GPg pg)    = pg
	
apply gs1 gs2 f = (new gs1) >>= \ps1 -> (new gs2) >>= \ps2 -> f ps1 ps2 >>= polset2geoset >>= return
	where
	polset2geoset gs = size gs >>= (polygonset2geometryset gs 0)		


foreign import stdcall unsafe "c_teoverlay" teoverlay :: TePolygonSetPtr -> TePolygonSetPtr ->  Int ->  Prelude.IO TePolygonSetPtr

	
	
-- metrics operations -------------------------
foreign import stdcall unsafe "c_tepoint_tedistance" tepoint_tedistance :: TePointPtr -> TePointPtr -> Prelude.IO Double
foreign import stdcall unsafe "c_telength" h_telength :: TeLine2DPtr ->  Prelude.IO Double
foreign import stdcall unsafe "c_tegeometryarea" tegeometryarea :: TePolygonPtr ->  Prelude.IO Double

foreign import stdcall unsafe "c_tepointset_tefindcentroid" tepointset_tefindcentroid :: TePointSetPtr ->  Prelude.IO TePointPtr
foreign import stdcall unsafe "c_tepolygonset_tefindcentroid" tepolygonset_tefindcentroid :: TePolygonSetPtr ->  Prelude.IO TePointPtr
foreign import stdcall unsafe "c_telineset_tefindcentroid" telineset_tefindcentroid :: TeLineSetPtr ->  Prelude.IO TePointPtr
foreign import stdcall unsafe "c_tepolygon_tefindcentroid" tepolygon_tefindcentroid :: TePolygonPtr ->  Prelude.IO TePointPtr

foreign import stdcall unsafe "c_set_precision" setPrecision :: Double ->  Prelude.IO ()

foreign import stdcall unsafe "c_tebufferregion" h_tebufferregion :: TePolygonPtr -> Double ->  Int ->  Prelude.IO TePolygonSetPtr

